<?php
/*
 * Plugin Name: WooCommerce MeoWallet
 * Plugin URI: https://developers.wallet.pt
 * Description: MeoWallet Payment Integration
 * Author: PT Pay, SA
 * Author URI: https://wallet.pt/
 * Version: 1.2.2
 * Text Domain: woocommerce-gateway-meowallet
 * Domain Path: /languages
 *
 * Copyright (c) 2016-2020 PT Pay, SA
 *
 */

include 'sdk/src/ApiResponse.php';
include 'sdk/src/Api.php';
include 'sdk/src/Encryption.php';
include 'sdk/src/exceptions/ApiResponseException.php';
include 'sdk/src/resources/ApiResource.php';
include 'sdk/src/resources/Checkout.php';
include 'sdk/src/resources/MB.php';
include 'sdk/src/resources/MBWAYRealtime.php';
include 'sdk/src/resources/Operations.php';
include 'sdk/src/resources/Wallet.php';
include 'sdk/src/structures/Address.php';
include 'sdk/src/structures/Client.php';
include 'sdk/src/structures/Item.php';

use PTPay\MeoWallet\SDK\Api as Api;
use PTPay\MeoWallet\SDK\Encryption;
use PTPay\MeoWallet\SDK\Resources\Checkout as Checkout;
use PTPay\MeoWallet\SDK\Resources\MBWAYRealtime;
use PTPay\MeoWallet\SDK\Resources\Wallet as Wallet;
use PTPay\MeoWallet\SDK\Resources\MB as MB;
use PTPay\MeoWallet\SDK\Structures\Client as Client;
use PTPay\MeoWallet\SDK\Structures\Item as Item;
use PTPay\MeoWallet\SDK\Structures\Address as Address;

function init_meowallet_gateway()
{



    add_action('admin_enqueue_scripts', 'enqueue_meowallet_admin_script');
    function enqueue_meowallet_admin_script() {
        wp_enqueue_script('meowallet-admin-script', plugins_url('assets/main.js', __FILE__), array('jquery'), null, true);
    }

    if (!class_exists('WC_Gateway_MeoWallet') && class_exists('WC_Payment_Gateway')) {


        class WC_Gateway_MeoWallet extends WC_Payment_Gateway
        {

            private static $instance = null;
            private $notices;
            public $supports = array('add_payment_method');
            const MBWAY_OPTION_INPLACE = 2;
            const MBWAY_OPTION_ENABLE = 1;

            public function getApiKey($isMB = null)
            {
                $env = $this->settings['environment'];
                if($isMB){
                    if($this->settings["enabled_two_wallets"] == "yes"){
                        return $this->decrypt($this->settings[$env."_api_token_mb"]);
                    }else {
                        return $this->decrypt($this->settings[$env."_api_token"]);
                    }
                }else {
                    return $this->decrypt($this->settings[$env."_api_token"]);
                }
                
            }

            public function decrypt($data)
            {
                return WC_MeoWallet::decrypt($data);
            }

            public function encrypt($data)
            {
                return WC_MeoWallet::encrypt($data);
            }

            protected function getMeoWalletEnv()
            {
                $env = isset($this->settings['environment']) ? $this->settings['environment'] : null;
                return $env;
            }

            protected function getApi($isMB = null)
            {
                return new Api($this->getApiKey($isMB), $this->getMeoWalletEnv());
            }

            protected function getCheckoutHTML()
            {
                require_once(dirname(__FILE__).DIRECTORY_SEPARATOR."views".DIRECTORY_SEPARATOR."checkout.php");

                return meowallet_checkout_html($this->settings);
            }

            public function get_description()
            {
                return $this->getCheckoutHTML();
            }

            public function processCallback($callback_data)
            {
                $callback_data = json_decode($callback_data);
                error_log("DEVLOG: Entrou para processar Callback em processCallback:");
                $checkout = new Checkout($this->getApi());

               if (true !== $checkout->verify($callback_data)) {

                   throw new InvalidArgumentException('Invalid callback');
               }

                if (!$callback_data) {
                    throw new Exception("Invalid JSON callback");
                }

                $order = new WC_Order($callback_data->ext_invoiceid);

                //TODOS - verificar o event se complet para mudar status da compra para o mesmo.
                $meowalletEventStatus = $callback_data->event;
                if($callback_data->operation_status === 'COMPLETED' && $meowalletEventStatus === 'COMPLETED')
                {
                    $order->update_status($this->settings['completed_order_status']);
                }else{
                    switch ($callback_data->operation_status) {
                        // case 'COMPLETED':
                        //     $order->update_status($this->settings['completed_order_status']);
                        //     break;
                        case 'FAILED':
                        case 'FAIL':
                            $order->update_status($this->settings['fail_order_status']);
                            break;
                    }
                }
             

                wc_reduce_stock_levels($order->get_id());
                die("OK");
            }

            public function meowallet_callback()
            {

                $callback_data = $this->_getRequestPayload();
	error_log("DEVLOG: Tentando para processar Callback em meowallet_callback:");

                $this->processCallback($callback_data);
            }

            protected function _getRequestPayload()
            {
                return file_get_contents('php://input');
            }

            public function getEmailHtml($order, $heading = false, $mailer)
            {

                $template = 'emails/customer-processing-order.php';

                return wc_get_template_html($template, array(
                    'order' => $order,
                    'email_heading' => $heading,
                    'sent_to_admin' => true,
                    'plain_text' => false,
                    'email' => $mailer
                ));
            }

            public function createMbRef()
            {

                if (current_user_can('manage_woocommerce')) {
                    $order_id = intval($_REQUEST['order_id']);
                    $order = new WC_Order($order_id);
                    $amount = round($order->get_total(), 2);

                    $url = admin_url('post.php?action=edit&post='.$order_id);
                    $mb = new MB($this->getApi(true));
                    $expire_date = (new DateTime('now'))->add(new DateInterval("P3D"));
                    try {


                        $checkout_data = $mb->createReference($amount, $order_id, 'EUR', $expire_date);

                        update_post_meta($order_id, "_meowallet_mb_checkout", $checkout_data);

                        $mb_data = [
                            'entity' => $checkout_data['mb']['entity'],
                            'ref' => $checkout_data['mb']['ref'],
                            'amount' => $checkout_data['amount'],
                            'expires' => $checkout_data['expires'],
                            'status' => $checkout_data['status']
                        ];
                        $order->add_order_note($this->getNoteContent($order_id, $mb_data));
                        $order->update_status($this->settings['pending_order_status']);


                        do_action('woocommerce_before_resend_order_emails', $order, 'customer_invoice');
                        // Send the customer invoice email.
                        WC()->payment_gateways();
                        WC()->shipping();
                        WC()->mailer()->customer_invoice($order);
                        $order->add_order_note(__('Order details manually sent to customer.', 'woocommerce'), false,
                            true);
                        do_action('woocommerce_after_resend_order_email', $order, 'customer_invoice');


                    } catch (Exception $e) {

                        error_log($e);
                        $_SESSION['meowallet_error'] = __('If you want create Multibanco references directly on your website, send an email to produto@wallet.pt',
                            WC_MeoWallet::TEXT_DOMAIN);
                        wp_send_json([
                            'result' => 'error',
                            'redirect' => $url
                        ]);
                    }

                    /*$mailer = WC()->mailer();
                    $recipient = $order->get_billing_email();
                    $subject = __('Multibanco reference created', WC_MeoWallet::TEXT_DOMAIN);
                    $content = $this->getEmailHtml($order, $subject, $mailer);
                    $headers = "Content-Type: text/html\r\n";
                    $mail_response = $mailer->send($recipient, $subject, $content, $headers);
*/


                    wp_send_json([
                        'result' => 'success',
                        'redirect' => $url
                    ]);
                    die();
                }

                return 0;
            }

            public function getNoteContent($order_id, $mb_data)
            {

                $arrRef = str_split($mb_data['ref'], 3);
                $ref = $arrRef[0]." ".$arrRef[1]." ".$arrRef[2];

                $msg = '';
                $msg .= __(" Entity: ", WC_MeoWallet::TEXT_DOMAIN);
                $msg .= $mb_data['entity']."<BR>";
                $msg .= __(" Reference: ", WC_MeoWallet::TEXT_DOMAIN);
                $msg .= $ref."<BR>";
                $msg .= __(" Amount: ", WC_MeoWallet::TEXT_DOMAIN);
                $msg .= number_format($mb_data['amount'], 2)." €<BR>";
                $msg .= __(" Expires at: ", WC_MeoWallet::TEXT_DOMAIN);
                $msg .= substr($mb_data['expires'], 0, 10)."<BR>";
                $msg .= __(" Estado: ", WC_MeoWallet::TEXT_DOMAIN);
                $msg .= __($mb_data['status'], WC_MeoWallet::TEXT_DOMAIN)."<BR>";

                return $msg;
            }

            public function getEmailMbData($order = null)
            {
                if (!$order) {
                    return;
                }
                $checkout_data = get_post_meta($order->get_id(), "_meowallet_mb_checkout");
                if (isset($checkout_data[0]) && $checkout_data[0]) {

                    $arrRef = str_split($checkout_data[0]['mb']['ref'], 3);
                    $ref = $arrRef[0]." ".$arrRef[1]." ".$arrRef[2];

                    $data = '
                    <h2 style="color: #557da1; display: block; font-family: "Helvetica Neue", Helvetica, Roboto, Arial, sans-serif; font-size: 18px; font-weight: bold; line-height: 130%; margin: 16px 0 8px; text-align: left;">'.__("Multibanco payment data",
                            WC_MeoWallet::TEXT_DOMAIN).'</h2>
                    <table border="0" cellspacing="0" cellpadding="6" style="width: 50%; font-family: \'Helvetica Neue\', Helvetica, Roboto, Arial, sans-serif;" border="1">

                        <tr>
                            <td class="td">'.__(" Entity: ", WC_MeoWallet::TEXT_DOMAIN).'</td>
                            <td class="td">'.$checkout_data[0]['mb']['entity'].'</td>
                        </tr>
                        <tr>
                            <td class="td">'.__(" Reference: ", WC_MeoWallet::TEXT_DOMAIN).'</td>
                            <td class="td">'.$ref.'</td>
                        </tr>
                        <tr>
                            <td class="td">'.__(" Amount: ", WC_MeoWallet::TEXT_DOMAIN).'</td>
                            <td class="td">'.number_format($checkout_data[0]['amount'], 2).' €</td>
                        </tr>
                        <tr>
                            <td class="td">'.__(" Expires at: ", WC_MeoWallet::TEXT_DOMAIN).'</td>
                            <td class="td">'.substr($checkout_data[0]['expires'], 0, 10).'</td>
                        </tr>
                    </table>';

                    echo $data;
                }
            }


            public function addNewMBBtn($order)
            {

                if ($order->get_status() != "pending") {
                    return;
                }

                ?>
                <script>
                    window.wordpress_root = '<?php echo site_url() ?>/';
                </script>

                <button data-order-id="<?= $order->get_id() ?>" type="button"
                        class="button button-primary create-mb-action"><?php _e('Generate Multibanco Reference',
                        WC_MeoWallet::TEXT_DOMAIN); ?></button>
                <?php
            }

            protected function registerAjaxCreateMbRef()
            {
                add_action('woocommerce_api_meowallet_createmb', array($this, 'createMbRef'));
            }

            protected function registerCallback()
            {
                add_action('woocommerce_api_meowallet_callback', array($this, 'meowallet_callback'));
            }

            public function registerOrderPageHooks()
            {
                if (is_admin()) {
                    add_action('woocommerce_order_item_add_action_buttons', array($this, 'addNewMBBtn'));
                    wp_enqueue_script('meowallet-order-page', plugins_url('assets/order-page.js', __FILE__));
                }

            }


            public function registerEmailHooks()
            {
                add_action('woocommerce_email_after_order_table', array($this, 'getEmailMbData'));
            }

            public function __construct()
            {
                $this->registerCallback();
                $this->registerAjaxCreateMbRef();
                $this->registerOrderPageHooks();
                $this->registerEmailHooks();
                $this->id = "meowallet";
                $this->icon = "https://wallet.pt/assets/images/favicon.ico";
                $this->method_title = "MEOWallet";
                $this->method_description = "The MEO Wallet is an innovative service, with personalized support, which allows you to accept payments on your store, online store or on TV, in a fast, convenient and secure way. All with competitive rates and no other costs. One service and one contract, based on security and confidentiality best practices, to support all essential payment methods - bank cards (debit and credit, VISA and Mastercard), MB References and MEO Wallet balance. Ensure customer satisfaction and enhance your business. Find more at wallet.pt. The MEO Wallet is a service of PT Pay, SA, registered payment institution at the Bank of Portugal under number 8705, belonging to PT Portugal.";
                $this->init_form_fields();
                $this->init_settings();
                $this->title = $this->get_option('title', "MEOWallet");


                $this->init_form_fields();

                add_action('woocommerce_update_options_payment_gateways_'.$this->id, array(
                    $this,
                    'process_admin_options'
                ));
                static::$instance = $this;
            }

            /**
             * @return WC_Gateway_MeoWallet | null
             */
            public static function getInstance()
            {
                if (!static::$instance) {
                    new self;
                }
                return static::$instance;
            }

            public function process_admin_options()
            {

                $post_data = $this->get_post_data();


                if ($post_data['woocommerce_meowallet_production_api_token'] != $this->settings['production_api_token']) {
                    $post_data['woocommerce_meowallet_production_api_token'] = $this->encrypt($post_data['woocommerce_meowallet_production_api_token']);
                } else {
                    $post_data['woocommerce_meowallet_production_api_token'] = $this->settings['production_api_token'];
                }
                if ($post_data['woocommerce_meowallet_sandbox_api_token'] != $this->settings['sandbox_api_token']) {
                    $post_data['woocommerce_meowallet_sandbox_api_token'] = $this->encrypt($post_data['woocommerce_meowallet_sandbox_api_token']);
                } else {
                    $post_data['woocommerce_meowallet_sandbox_api_token'] = $this->settings['sandbox_api_token'];
                }

                //Mbway/Multibanco API Key (Production)
                  if ($post_data['woocommerce_meowallet_production_api_token_mb'] != $this->settings['production_api_token_mb']) {
                    $post_data['woocommerce_meowallet_production_api_token_mb'] = $this->encrypt($post_data['woocommerce_meowallet_production_api_token_mb']);
                } else {
                    $post_data['woocommerce_meowallet_production_api_token_mb'] = $this->settings['production_api_token_mb'];
                }
                 //Mbway/Multibanco API Key (Sandbox)
                if ($post_data['woocommerce_meowallet_sandbox_api_token_mb'] != $this->settings['sandbox_api_token_mb']) {
                    $post_data['woocommerce_meowallet_sandbox_api_token_mb'] = $this->encrypt($post_data['woocommerce_meowallet_sandbox_api_token_mb']);
                } else {
                    $post_data['woocommerce_meowallet_sandbox_api_token_mb'] = $this->settings['sandbox_api_token_mb'];
                }

                $this->set_post_data($post_data);

                return parent::process_admin_options();
            }

            public function init_form_fields()
            {
                $this->form_fields = array(
                    'enabled' => array(
                        'title' => __('Enable/Disable', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'checkbox',
                        'label' => __('Enable MEOWallet Payment', WC_MeoWallet::TEXT_DOMAIN),
                        'default' => 'yes'
                    ),
                    'title' => array(
                        'title' => __('Title', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'text',
                        'description' => __('This controls the title which the user sees during checkout.',
                            WC_MeoWallet::TEXT_DOMAIN),
                        'default' => __('MEOWallet', WC_MeoWallet::TEXT_DOMAIN),
                        'desc_tip' => true,
                    ),
                    'description' => array(
                        'title' => __('Customer Message', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'textarea',
                        'default' => 'MeoWallet',
                    ),
                    'environment' => array(
                        'title' => __('Environment', WC_MeoWallet::TEXT_DOMAIN),
                        'label' => __('Environment', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'select',
                        'options' => array(
                            'production' => __('Production', WC_MeoWallet::TEXT_DOMAIN),
                            'sandbox' => __('Sandbox', WC_MeoWallet::TEXT_DOMAIN)
                        ),
                    ),
                    'enabled_two_wallets' => array(
                        'title' => __('Enable/Disable use two wallets', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'checkbox',
                        'label' => __('Enable to receive in a different Wallet payments of type MBway or Multibanco reference', WC_MeoWallet::TEXT_DOMAIN),
                        'default' => 'no',
                        'description' => __('More info https://developers.wallet.pt/en/basics/auth.html',
                            WC_MeoWallet::TEXT_DOMAIN),
                    ),
                    'production_api_token' => array(
                        'title' => __('Production API Token', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'password',
                        'description' => __('More info https://developers.wallet.pt/en/basics/auth.html',
                            WC_MeoWallet::TEXT_DOMAIN),
                    ),
                    'production_api_token_mb' => array(
                        'title' => __('Production API Token - MB', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'password',
                        'description' => __('More info https://developers.wallet.pt/en/basics/auth.html',
                            WC_MeoWallet::TEXT_DOMAIN),
                    ),
                    'sandbox_api_token' => array(
                        'title' => __('Sandbox API Token', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'text',
                        'description' => __('More info https://developers.wallet.pt/en/basics/auth.html',
                            WC_MeoWallet::TEXT_DOMAIN),
                    ),
                    'sandbox_api_token_mb' => array(
                        'title' => __('Sandbox API Token - MB', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'text',
                        'description' => __('More info https://developers.wallet.pt/en/basics/auth.html',
                            WC_MeoWallet::TEXT_DOMAIN),
                    ),
                    'pending_order_status' => array(
                        'title' => __('Pending Payment Order Status', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'select',
                        'options' => wc_get_order_statuses(),
                    ),
                    'completed_order_status' => array(
                        'title' => __('Completed Payment Order Status', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'select',
                        'options' => wc_get_order_statuses(),
                    ),
                    'fail_order_status' => array(
                        'title' => __('Failed Payment Order Status', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'select',
                        'options' => wc_get_order_statuses(),
                    ),
                    'method_meowallet' => array(
                        'title' => __('MEOWallet', WC_MeoWallet::TEXT_DOMAIN),
                        'label' => __('Payment Method', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'select',
                        'options' => array(
                            '1' => __('Enabled', WC_MeoWallet::TEXT_DOMAIN),
                            '0' => __('Disabled', WC_MeoWallet::TEXT_DOMAIN),
                        ),
                    ),
                    'method_CC' => array(
                        'title' => __('Credit Card', WC_MeoWallet::TEXT_DOMAIN),
                        'label' => __('Payment Method', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'select',
                        'options' => array(
                            '1' => __('Enabled', WC_MeoWallet::TEXT_DOMAIN),
                            '0' => __('Disabled', WC_MeoWallet::TEXT_DOMAIN),
                        ),
                    ),
                    'method_MB' => array(
                        'title' => __('Multibanco', WC_MeoWallet::TEXT_DOMAIN),
                        'label' => __('Payment Method', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'select',
                        'options' => array(
                            '1' => __('Enabled', WC_MeoWallet::TEXT_DOMAIN),
                            '0' => __('Disabled', WC_MeoWallet::TEXT_DOMAIN),
                        ),
                    ),
                    'method_PAYPAL' => array(
                        'title' => __('PayPal', WC_MeoWallet::TEXT_DOMAIN),
                        'label' => __('Payment Method', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'select',
                        'options' => array(
                            '1' => __('Enabled', WC_MeoWallet::TEXT_DOMAIN),
                            '0' => __('Disabled', WC_MeoWallet::TEXT_DOMAIN),
                        ),
                        'description' => __('The chosen methods here must be the same as agreed with PT Pay, and you can see them on your account at wallet.pt',
                            WC_MeoWallet::TEXT_DOMAIN)
                    ),
                    'method_MBWAY' => array(
                        'title' => __('MBWay', WC_MeoWallet::TEXT_DOMAIN),
                        'label' => __('Payment Method', WC_MeoWallet::TEXT_DOMAIN),
                        'type' => 'select',
                        'options' => array(
                            static::MBWAY_OPTION_INPLACE => __('In Place', WC_MeoWallet::TEXT_DOMAIN),
                            static::MBWAY_OPTION_ENABLE => __('Enabled', WC_MeoWallet::TEXT_DOMAIN),
                            '0' => __('Disabled', WC_MeoWallet::TEXT_DOMAIN),
                        ),
                    ),
                );


            }

            protected function createCheckOutItems(WC_Order $order)
            {
                $checkout_items = [];
                $items = $order->get_items();
                foreach ($items as $item) {
                    $product = new WC_Product($item['product_id']);

                    $checkout_item = new Item();

                    $checkout_item->setName($item['name']);
                    $checkout_item->setDescription($product->get_title());
                    $checkout_item->setQuantity($item['qty']);
                    $checkout_item->setRef($product->get_sku());
                    $checkout_items[] = $checkout_item;
                }

                return $checkout_items;
            }

            public function validateMbWayPhone()
            {

                $n = null;
                if (isset($_REQUEST['mbway-phone'])
                    && strlen($_REQUEST['mbway-phone']) == 9
                    && preg_match('/^(9[1236])[0-9]{7}$/', $_REQUEST['mbway-phone'], $n)) {
                    $mbway_phone = $_REQUEST['mbway-phone'];
                } else {
                    return false;
                }
                return $mbway_phone;
            }
            public function process_payment($order_id)
            {
                $order = new WC_Order($order_id);
                if ($order->get_currency() != "EUR") {
                    wc_add_notice(__("Currency not supported.", WC_MeoWallet::TEXT_DOMAIN), 'error');
                    return;
                }
                // Handle payment
                if ($order->get_total() > 0) {


                    $checkout_items = $this->createCheckOutItems($order);

                    $address = new Address();
                    $address->setCountry($order->get_billing_country());
                    $address->setAddress($order->get_billing_address_1()." ".$order->get_billing_address_2());
                    $address->setCity($order->get_billing_city());
                    $address->setPostalcode($order->get_billing_postcode());

                    $client = new Client;
                    $client->setName($order->get_billing_first_name()." ".$order->get_billing_last_name());
                    $client->setEmail($order->get_billing_email());
                    $client->setAddress($address);

                    
                    $isMb = $_REQUEST['meowallet_method']  == "MB" || $_REQUEST['meowallet_method']  == "MBWAYRT" ? true : false ;
                
                    $api = new Api($this->getApiKey($isMb), $this->getMeoWalletEnv());
                    
                    $amount = round($order->get_total(), 2);
                    try { //create MB references
                        if ($_REQUEST['meowallet_method'] == "MB") {
                            return array(
                                'result' => 'success',
                                'redirect' => $this->paymentByMMRef($amount, $order_id)
                            );
                        }
                    } catch (Exception $e) {
                        error_log($e->getMessage());
                        //if fail... "web checkout"
                    }


                    try {
                        if ($_REQUEST['meowallet_method'] == "MBWAYRT") {
                            if (!$mbphone = $this->validateMbWayPhone()) {

                                $message = __('Invalid MBWay phone number', WC_MeoWallet::TEXT_DOMAIN);
                                wc_add_notice($message, 'error');
                                //wp_safe_redirect(wp_get_referer());

                                $data = array(
                                    'error' => true,
                                    'result' => "failure",
                                    'redirect_' => wc_get_checkout_url(),
                                    'messages' => $message
                                );
                                return $data;
                            }
                            $url = $this->createMBWAYRTRequest($client, $checkout_items, $amount, $order, $order_id,
                                $mbphone);
                            return array(
                                'result' => 'success',
                                'redirect' => $url
                            );
                        }
                    } catch (Exception $e) {
                        error_log($e->getMessage());
                    }

                    $checkout = new Checkout($api);

                    try {
                        $checkout_data = $checkout->create(
                            $client, $checkout_items, $amount, $order->get_currency(),
                            $order->get_checkout_order_received_url(), $order->get_cancel_endpoint(),
                            $order->get_billing_email(), $order_id."", $order->get_billing_email(),
                            $_REQUEST['meowallet_method']
                        );
                    } catch (PTPay\MeoWallet\SDK\Exceptions\ApiResponseException $e) {
                        wc_add_notice(__($e->getReason(), WC_MeoWallet::TEXT_DOMAIN), 'error');

                        return;
                    }
                    do_action('meowallet_new_checkout', $checkout_data);
                    update_post_meta($order_id, "_meowallet_checkout", $checkout_data);
                    $url = $checkout_data['url_redirect'];
                } else {
                    $order->payment_complete();
                }
                WC()->cart->empty_cart();

                return array(
                    'result' => 'success',
                    'redirect' => $url
                );
            }

            /**
             * @param $amount
             * @param $order_id
             * @return array
             */
            protected function paymentByMMRef($amount, $order_id)
            {
                $mb = new MB($this->getApi(true));
                $expire_date = (new DateTime('now'))->add(new DateInterval("P3D"));
                $checkout_data = $mb->createReference($amount, $order_id, 'EUR', $expire_date);
                update_post_meta($order_id, "_meowallet_mb_checkout", $checkout_data);
                $url = (get_permalink(get_page_by_path(WC_MeoWallet::MB_DATA_PAGE_SLUG)));
                do_action('meowallet_new_mb', $checkout_data);
                WC()->cart->empty_cart();
                $_SESSION['_meowallet_mb_checkout'] = $checkout_data;
                $order = new WC_Order($order_id);

                do_action('woocommerce_before_resend_order_emails', $order, 'customer_invoice');
                // Send the customer invoice email.
                WC()->payment_gateways();
                WC()->shipping();
                WC()->mailer()->customer_invoice($order);
                $order->add_order_note(__('Order details manually sent to customer.', 'woocommerce'), false, true);
                do_action('woocommerce_after_resend_order_email', $order, 'customer_invoice');

                return $url;
            }

            /**
             * @param  Client  $client
             * @param  array  $checkout_items
             * @param $amount
             * @param  WC_Order  $order
             * @param $order_id
             * @return bool|false|string|WP_Error
             */
            protected function createMBWAYRTRequest(
                Client $client,
                array $checkout_items,
                $amount,
                WC_Order $order,
                $order_id,
                $mbway_phone
            ) {
                $mbway = new MBWAYRealtime($this->getApi(true));


                $response = $mbway->create($client,
                    $checkout_items,
                    $amount,
                    'EUR',
                    $order->get_checkout_order_received_url(),
                    $order->get_cancel_endpoint(),
                    $order->get_billing_email(),
                    $order_id."",
                    $client->getEmail(), $mbway_phone);

                $_SESSION['_meowallet_mbway_checkout'] = $response;
                update_post_meta($order_id, "_meowallet_mbway_checkout", $response);
                $url = (get_permalink(get_page_by_path(WC_MeoWallet::MBWAY_DATA_PAGE_SLUG)));
                do_action('meowallet_new_mbway', $response);

                WC()->cart->empty_cart();
                return $url;
            }
        }

    }
}

class WC_MeoWallet
{

    const PLUGIN_DOMAIN = "woocommerce-gateway-meowallet";

    const TEXT_DOMAIN = "meowallet";
    const MB_DATA_PAGE_SLUG = 'meowallet-mb-data';
    const MB_DATA_PAGE_SLUG_CALLBACK = 'meowallet-mb-callback';

    const MBWAY_DATA_PAGE_SLUG = 'meowallet-mbway-data';


    protected $cipher;

    public function __construct()
    {
        $this->init();
        $this->installPages();
        if (!session_id()) {
            @session_start();
        }
        add_action('init', array($this, 'load_plugin_textdomain'));
        add_action('admin_notices', array($this, 'displayErrors'));

    }

    public function init()
    {

        add_action('plugins_loaded', 'init_meowallet_gateway');

        add_filter('woocommerce_payment_gateways', function ($methods) {

            $methods[] = 'WC_Gateway_MeoWallet';

            return $methods;
        });
        add_shortcode('woocommerce_meowallet_mb_data', array($this, 'getMultibancoData'));
        add_shortcode('woocommerce_meowallet_mbway_data', array($this, 'getMBWayData'));
        add_shortcode('woocommerce_meowallet_mb_callback_sucess', array($this, 'getMultibancoCallBacksucess'));
        add_shortcode('woocommerce_meowallet_mb_callback_fail', array($this, 'getMultibancoCallBackfail'));
        add_action('woocommerce_email_after_order_table', array($this, 'getMultibancoCallBackfail'));
        add_action('wp_enqueue_scripts', array($this, 'addCheckoutScript'));
    }

    public static function addCheckoutScript()
    {
        if (get_option('woocommerce_checkout_page_id') == get_the_ID()) {

            $plugin_js_path = plugins_url('assets/input-checked.js', __FILE__);
            wp_register_script('meowallet-script-input', $plugin_js_path, array('jquery'), '1.0', true);
            wp_enqueue_script('meowallet-script-input');
        }
    }

    public function installPages()
    {
        $mb_data_page = get_posts([
            'name' => self::MB_DATA_PAGE_SLUG,
            'post_type' => 'page',
            'post_status' => 'publish',
        ]);

        if (!isset($mb_data_page[0])) {

            $mb_data_page = array(
                'post_title' => __('Multibanco data', WC_MeoWallet::TEXT_DOMAIN),
                'post_content' => '[woocommerce_meowallet_mb_data]',
                'post_status' => 'publish',
                'post_type' => 'page',
                'post_name' => self::MB_DATA_PAGE_SLUG,
                'post_author' => 1,
            );
            wp_insert_post($mb_data_page);
        }
        $mbway_page = get_posts([
            'name' => self::MBWAY_DATA_PAGE_SLUG,
            'post_type' => 'page',
            'post_status' => 'publish',
        ]);

        if (!isset($mbway_page[0])) {

            $mbway_page = array(
                'post_title' => __('Dados MBWay', WC_MeoWallet::TEXT_DOMAIN),
                'post_content' => '[woocommerce_meowallet_mbway_data]',
                'post_status' => 'publish',
                'post_type' => 'page',
                'post_name' => self::MBWAY_DATA_PAGE_SLUG,
                'post_author' => 1,
            );
            wp_insert_post($mbway_page);
        }

    }

    public static function decrypt($data)
    {
        $enc = new Encryption(AUTH_KEY);
        return $enc->decrypt($data);

    }

    public static function encrypt($data)
    {
        $enc = new Encryption(AUTH_KEY);
        return $enc->encrypt($data);
    }

    public function getMBWayData()
    {
        if (isset($_SESSION['_meowallet_mbway_checkout'])) {

            $order_id = $_SESSION['_meowallet_mbway_checkout']['ext_invoiceid'];

            $order = wc_get_order($order_id);
            $order_data = $order->get_data();


            if (is_array($order_data) && count($order_data) > 1) {
                $out = '';
                $status = $order_data['status'];

                $meowallet_settings = WC_Gateway_MeoWallet::getInstance()->settings;
                $meowallet_settings = str_replace("wc-", "", $meowallet_settings);


                $class = "info";
                $text = "pending";
                switch ($status) {
                    case $meowallet_settings['completed_order_status']:
                        $class = "message";
                        $text = "processing";
                        break;
                    case $meowallet_settings['fail_order_status']:
                        $class = "error";
                        $text = "cancelled";
                        break;
                }

                $out = $out.'<div class="woocommerce-'.$class.' meowallet meowallet-mbway-status meowallet-mbway-status-'.$status.'">
                       '.__($text, WC_MeoWallet::TEXT_DOMAIN).'
                       </div>
                ';

                if ($status == "pending") {
                    $out = $out.'
                        <script>
                            setTimeout(function() {
                                location.reload();
                            },7500);
                        </script>';
                }
                if ($status == $meowallet_settings['completed_order_status']) {
                    $out = $out.'<script>
                        setTimeout(function(){
                            window.location.href = \''.$order->get_checkout_order_received_url().'\';
                        }, 5000);
                    </script>
                    ';
                }
                return $out;
            }

        }
    }

    public function getMultibancoData()
    {

        if (!isset($_SESSION['_meowallet_mb_checkout'])) {
            return;
        }

        $data = $_SESSION['_meowallet_mb_checkout'];

        $url = (get_permalink(get_page_by_path(WC_MeoWallet::MB_DATA_PAGE_SLUG)));
        $url = str_replace("meowallet-mb-data/", "", $url);

        $data['mb']['ref'] = implode(' ', str_split($data['mb']['ref'], 3));
        $expires = new DateTime($data['expires']);
        $data['expires'] = $expires->format('Y-m-d');

        $ref = ($data['mb']['ref']);

        global $woocommerce;
        if (!$woocommerce) return;
        $cart_url = $woocommerce->cart->get_cart_url();


        return '<table>
                  <tbody>
                  <tr>
                    <td class="mb_data_title">'.__('Entity', WC_MeoWallet::TEXT_DOMAIN).'</td>
                    <td class="mb_entity">'.$data['mb']['entity'].'</td>
                  </tr>
                  <tr>
                    <td class="mb_data_title">'.__('Reference', WC_MeoWallet::TEXT_DOMAIN).'</td>
                    <td class="mb_reference">'.$ref.'</td>
                  </tr>
                  <tr>
                    <td class="mb_data_title">'.__('Amount', WC_MeoWallet::TEXT_DOMAIN).'</td>
                    <td class="mb_value">'.$data['amount'].' EUR</td>
                  </tr>
                  <tr>
                    <td class="mb_data_title">'.__('Expires at', WC_MeoWallet::TEXT_DOMAIN).'</td>
                    <td class="mb_expires">'.$data['expires'].'</td>
                  </tr>
                  </tbody>
                </table>
                <BR>
                <a href="'.$cart_url.'" >Voltar</a>
                ';


    }

    public function displayErrors()
    {

        if (isset($_SESSION['meowallet_error']) && $_SESSION['meowallet_error']) {
            ?>
            <div class="notice notice-error is-dismissible">
                <p><?php echo $_SESSION['meowallet_error']; ?></p>
            </div>
            <?php


        }
        $_SESSION['meowallet_error'] = false;
    }

    function load_plugin_textdomain()
    {
        $domain = WC_MeoWallet::TEXT_DOMAIN;
        $mo_file = __DIR__.'/languages/'.WC_MeoWallet::TEXT_DOMAIN.'-'.get_locale().'.mo';
        if (file_exists($mo_file)) {

            load_textdomain($domain, $mo_file);
            load_plugin_textdomain(WC_MeoWallet::TEXT_DOMAIN, false, dirname(plugin_basename(__FILE__)).'/languages/');
        } else {
            error_log("Fail to load: ".$mo_file);
        }

    }

    public function installMbCallBack()
    {
        $page = get_posts([
            'name' => 'meowallet-mb-callback',
            'post_type' => 'page',
            'post_status' => 'publish',
        ]);


        if (!isset($page[0])) {

            $my_post = array(
                'post_title' => __('Multibanco Callback', WC_MeoWallet::TEXT_DOMAIN),
                'post_content' => '[woocommerce_meowallet_mb_callback]',
                'post_status' => 'publish',
                'post_type' => 'page',
                'post_name' => self::MB_DATA_PAGE_SLUG_CALLBACK,
                'post_author' => 1,
            );
            wp_insert_post($my_post);
        }
    }

    public function getMultibancoCallBackfail()
    {

        $url = (get_permalink(get_page_by_path(WC_MeoWallet::MB_DATA_PAGE_SLUG_CALLBACK)));
        $data['status'] = 'VOIDED';

        $url = str_replace("callbackfail/", "", $url);

        return '<BR>
                <a href="'.$url.'" >Voltar</a>
                ';


    }

    public function getMultibancoCallBacksucess()
    {

        $url = (get_permalink(get_page_by_path(WC_MeoWallet::MB_DATA_PAGE_SLUG_CALLBACK)));

        $url = str_replace("callbackok/", "", $url);

        return '<BR><a href="'.$url.'" >Voltar</a>
                ';

    }
}

new WC_MeoWallet();


